Attribute VB_Name = "MClipView"
' *************************************************************
'  Copyright (C)1995-98, Karl E. Peterson
'  Author grants royalty-free rights to use this code within
'  compiled applications. Selling or otherwise distributing
'  this source code is not allowed without author's express
'  permission.
' *************************************************************
Option Explicit

' Required data structures
Public Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Public Type RECTL
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Type SIZEL
   cx As Long
   cy As Long
End Type

Public Type PAINTSTRUCT
   hdc As Long
   fErase As Long
   rcPaint As RECT
   fRestore As Long
   fIncUpdate As Long
   rgbReserved(1 To 32) As Byte
End Type

Private Type POINTAPI
   x As Long
   y As Long
End Type

' Clipboard Manager Functions
Public Declare Function OpenClipboard Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function CloseClipboard Lib "user32" () As Long
Public Declare Function GetClipboardOwner Lib "user32" () As Long
Public Declare Function SetClipboardViewer Lib "user32" (ByVal hWnd As Long) As Long
Public Declare Function GetClipboardViewer Lib "user32" () As Long
Public Declare Function ChangeClipboardChain Lib "user32" (ByVal hWnd As Long, ByVal hWndNext As Long) As Long
Public Declare Function SetClipboardData Lib "user32" (ByVal wFormat As Long, ByVal hMem As Long) As Long
Public Declare Function GetClipboardData Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function RegisterClipboardFormat Lib "user32" Alias "RegisterClipboardFormatA" (ByVal lpString As String) As Long
Public Declare Function CountClipboardFormats Lib "user32" () As Long
Public Declare Function EnumClipboardFormats Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function GetClipboardFormatName Lib "user32" Alias "GetClipboardFormatNameA" (ByVal wFormat As Long, ByVal lpString As String, ByVal nMaxCount As Long) As Long
Public Declare Function EmptyClipboard Lib "user32" () As Long
Public Declare Function IsClipboardFormatAvailable Lib "user32" (ByVal wFormat As Long) As Long
Public Declare Function GetPriorityClipboardFormat Lib "user32" (lpPriorityList As Long, ByVal nCount As Long) As Long
Public Declare Function GetOpenClipboardWindow Lib "user32" () As Long

' Other required Win32 APIs
Public Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Long) As Long
Public Declare Function GlobalAlloc Lib "kernel32" (ByVal wFlags As Long, ByVal dwBytes As Long) As Long
Public Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long
Public Declare Function GlobalHandle Lib "kernel32" (wMem As Any) As Long
Public Declare Function GlobalLock Lib "kernel32" (ByVal hMem As Long) As Long
Public Declare Function GlobalSize Lib "kernel32" (ByVal hMem As Long) As Long
Public Declare Function GlobalUnlock Lib "kernel32" (ByVal hMem As Long) As Long
Public Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)
Public Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function SetMapMode Lib "gdi32" (ByVal hdc As Long, ByVal nMapMode As Long) As Long
Private Declare Function SetWindowOrgEx Lib "gdi32" (ByVal hdc As Long, ByVal nX As Long, ByVal nY As Long, lpPoint As Any) As Long
Private Declare Function SetWindowExtEx Lib "gdi32" (ByVal hdc As Long, ByVal nX As Long, ByVal nY As Long, lpSize As Any) As Long
Private Declare Function SetViewportExtEx Lib "gdi32" (ByVal hdc As Long, ByVal nX As Long, ByVal nY As Long, lpSize As Any) As Long
Private Declare Function SetViewportOrgEx Lib "gdi32" (ByVal hdc As Long, ByVal nX As Long, ByVal nY As Long, lpPoint As Any) As Long
Private Declare Function PlayEnhMetaFile Lib "gdi32" (ByVal hdc As Long, ByVal hEmf As Long, lpRect As RECT) As Long
Private Declare Function PlayMetaFile Lib "gdi32" (ByVal hdc As Long, ByVal hMF As Long) As Long
Private Declare Function GetMetaFileBitsEx Lib "gdi32" (ByVal hMF As Long, ByVal nSize As Long, lpvData As Any) As Long
Private Declare Function GetEnhMetaFileHeader Lib "gdi32" (ByVal hEmf As Long, ByVal cbBuffer As Long, lpemh As ENHMETAHEADER) As Long
Private Declare Function DragQueryFile Lib "shell32.dll" Alias "DragQueryFileA" (ByVal hDrop As Long, ByVal UINT As Long, ByVal lpStr As String, ByVal ch As Long) As Long
Private Declare Function DragQueryPoint Lib "shell32.dll" (ByVal hDrop As Long, lpPoint As POINTAPI) As Long

' Clipboard Window Messages
Public Const WM_CUT = &H300
Public Const WM_COPY = &H301
Public Const WM_PASTE = &H302
Public Const WM_CLEAR = &H303
Public Const WM_UNDO = &H304
Public Const WM_RENDERFORMAT = &H305
Public Const WM_RENDERALLFORMATS = &H306
Public Const WM_DESTROYCLIPBOARD = &H307
Public Const WM_DRAWCLIPBOARD = &H308
Public Const WM_PAINTCLIPBOARD = &H309
Public Const WM_VSCROLLCLIPBOARD = &H30A
Public Const WM_SIZECLIPBOARD = &H30B
Public Const WM_ASKCBFORMATNAME = &H30C
Public Const WM_CHANGECBCHAIN = &H30D
Public Const WM_HSCROLLCLIPBOARD = &H30E

' Predefined Clipboard Formats
Public Const CF_TEXT = 1
Public Const CF_BITMAP = 2
Public Const CF_METAFILEPICT = 3
Public Const CF_SYLK = 4
Public Const CF_DIF = 5
Public Const CF_TIFF = 6
Public Const CF_OEMTEXT = 7
Public Const CF_DIB = 8
Public Const CF_PALETTE = 9
Public Const CF_PENDATA = 10
Public Const CF_RIFF = 11
Public Const CF_WAVE = 12
Public Const CF_UNICODETEXT = 13
Public Const CF_ENHMETAFILE = 14
Public Const CF_HDROP = 15
Public Const CF_LOCALE = 16
Public Const CF_MAX = 17

Public Const CF_OWNERDISPLAY = &H80
Public Const CF_DSPTEXT = &H81
Public Const CF_DSPBITMAP = &H82
Public Const CF_DSPMETAFILEPICT = &H83
Public Const CF_DSPENHMETAFILE = &H8E

' "Public" formats don't get GlobalFree()'d
Public Const CF_PublicFIRST = &H200
Public Const CF_PublicLAST = &H2FF

' "GDIOBJ" formats do get DeleteObject()'d
Public Const CF_GDIOBJFIRST = &H300
Public Const CF_GDIOBJLAST = &H3FF

' Global Memory Flags
Public Const GMEM_FIXED = &H0
Public Const GMEM_MOVEABLE = &H2
Public Const GMEM_NOCOMPACT = &H10
Public Const GMEM_NODISCARD = &H20
Public Const GMEM_ZEROINIT = &H40
Public Const GMEM_MODIFY = &H80
Public Const GMEM_DISCARDABLE = &H100
Public Const GMEM_NOT_BANKED = &H1000
Public Const GMEM_SHARE = &H2000
Public Const GMEM_DDESHARE = &H2000
Public Const GMEM_NOTIFY = &H4000
Public Const GMEM_LOWER = GMEM_NOT_BANKED
Public Const GMEM_VALID_FLAGS = &H7F72
Public Const GMEM_INVALID_HANDLE = &H8000

Public Const GHND = (GMEM_MOVEABLE Or GMEM_ZEROINIT)
Public Const GPTR = (GMEM_FIXED Or GMEM_ZEROINIT)

' Flags returned by GlobalFlags (in addition to GMEM_DISCARDABLE)
Public Const GMEM_DISCARDED = &H4000
Public Const GMEM_LOCKCOUNT = &HFF

' Metafile structures
Private Type METAFILEPICT
   mm As Long
   xExt As Long
   yExt As Long
   hMF As Long
End Type

Private Type ENHMETAHEADER
   iType As Long
   nSize As Long
   rclBounds As RECT
   rclFrame As RECT
   dSignature As Long
   nVersion As Long
   nBytes As Long
   nRecords As Long
   nHandles As Integer
   sReserved As Integer
   nDescription As Long
   offDescription As Long
   nPalEntries As Long
   szlDevice As SIZEL
   szlMillimeters As SIZEL
End Type


' Mapping Modes
Private Const MM_TEXT = 1
Private Const MM_LOMETRIC = 2
Private Const MM_HIMETRIC = 3
Private Const MM_LOENGLISH = 4
Private Const MM_HIENGLISH = 5
Private Const MM_TWIPS = 6
Private Const MM_ISOTROPIC = 7
Private Const MM_ANISOTROPIC = 8

' ****************************************************
'  Clipboard manipulation routines (public)
' ****************************************************
Public Function CountClipFormats() As Long
   CountClipFormats = CountClipboardFormats
End Function

Public Function DisplayFileList(txt As TextBox) As Boolean
   Dim sf As New CStockFont
   Dim hDrop As Long
   Dim nFiles As Long
   Dim i As Long
   Dim desc As String
   Dim filename As String
   Const MAX_PATH = 260
   
   Set sf.Client = txt
   sf.StockFont = sfSystemFixed
   filename = Space(MAX_PATH)
   
   If OpenClipboard(0&) Then
      hDrop = GetClipboardData(CF_HDROP)
      nFiles = DragQueryFile(hDrop, -1&, "", 0)
      For i = 0 To nFiles - 1
         Call DragQueryFile(hDrop, i, filename, Len(filename))
         desc = desc & TrimNull(filename) & vbCrLf
      Next i
      txt.Text = desc
      Call CloseClipboard
      DisplayFileList = True
      txt.Parent.UpdateCaption "CF_HDROP"
   End If
End Function

Public Function DisplayLocale(txt As TextBox) As Boolean
   Dim sf As New CStockFont
   Dim msg As String
   Dim nRet As Long
   Dim LCID As Long
   
   Set sf.Client = txt
   sf.StockFont = sfAnsiFixed
   If OpenClipboard(0&) Then
      nRet = GetClipboardData(CF_LOCALE)
      CopyMem LCID, ByVal nRet, 4
      txt.Text = LocaleDescLong(LCID)
      Call CloseClipboard
      DisplayLocale = True
      txt.Parent.UpdateCaption "CF_LOCALE"
   End If
End Function

Public Function DisplayText(ByVal nFmt As Long, txt As TextBox) As Boolean
   Dim sf As New CStockFont
   txt.Text = Clipboard.GetText()
   Set sf.Client = txt
   Select Case nFmt
      Case CF_TEXT
         sf.StockFont = sfAnsiVar
         txt.Parent.UpdateCaption "CF_TEXT"
      Case CF_OEMTEXT
         sf.StockFont = sfOemFixed
         txt.Parent.UpdateCaption "CF_OEMTEXT"
      Case CF_DSPTEXT
         sf.StockFont = sfDeviceDefault
         txt.Parent.UpdateCaption "CF_DSPTEXT"
   End Select
   DisplayText = True
End Function

Public Function DisplayPicture(ByVal nFmt As Long, pic As PictureBox) As Boolean
   Select Case nFmt
      Case CF_ENHMETAFILE
         DisplayPicture = PaintEnhMetafile(pic)
         pic.Parent.UpdateCaption "CF_ENHMETAFILE"
      Case CF_METAFILEPICT
         DisplayPicture = PaintMetafile(pic)
         pic.Parent.UpdateCaption "CF_METAFILEPICT"
      Case CF_BITMAP
         pic.Picture = Clipboard.GetData(CF_BITMAP)
         pic.Parent.UpdateCaption "CF_BITMAP"
         DisplayPicture = True
      Case CF_DSPBITMAP
         pic.Picture = Clipboard.GetData(CF_BITMAP)
         pic.Parent.UpdateCaption "CF_DSPBITMAP"
         DisplayPicture = True
      Case CF_DSPMETAFILEPICT
         DisplayPicture = PaintMetafile(pic)
         pic.Parent.UpdateCaption "CF_DSPMETAFILEPICT"
   End Select
End Function

Public Function DisplayPalette(pic As PictureBox) As Boolean
   DisplayPalette = PaintPalette(pic)
   pic.Parent.UpdateCaption "CF_PALETTE"
End Function

Public Function DisplayOwnerDisplay(pic As PictureBox) As Boolean
   Dim hWndOwner As Long
   Dim hGlb As Long
   Dim rClient As RECT
   Dim ps As PAINTSTRUCT
   Dim lpps As Long
   Dim nRet As Long
   
   Call GetClientRect(pic.hWnd, rClient)
   hGlb = GlobalAlloc(GMEM_FIXED Or GMEM_ZEROINIT, Len(rClient))
   nRet = GlobalLock(hGlb)
   Call CopyMem(nRet, rClient, Len(rClient))
   hWndOwner = GetClipboardOwner()
   Call SendMessage(hWndOwner, WM_SIZECLIPBOARD, pic.Parent.hWnd, ByVal hGlb)
   Call GlobalUnlock(hGlb)
   Call GlobalFree(hGlb)
   
   hGlb = GlobalAlloc(GMEM_DDESHARE, Len(ps))
   lpps = GlobalLock(hGlb)
   
   ps.hdc = pic.hdc
   ps.fErase = True
   ps.rcPaint = rClient
   hGlb = GlobalAlloc(GMEM_FIXED Or GMEM_ZEROINIT, Len(ps))
   nRet = GlobalLock(hGlb)
   Call CopyMem(nRet, ps, Len(ps))
   Call SendMessage(hWndOwner, WM_PAINTCLIPBOARD, pic.Parent.hWnd, ByVal hGlb)
   Call GlobalUnlock(hGlb)
   Call GlobalFree(hGlb)
   
   pic.Parent.UpdateCaption "CF_OWNERDRAW"
   DisplayOwnerDisplay = True
End Function

Public Sub UpdateDisplayMenu(mnuArray As Variant)
   Dim mnu As Menu
   Dim nCnt As Long
   Dim nFmt As Long
   Dim sFmt As String
   Dim nRet As Long
   Dim i As Long
   '
   ' Clear previous menu array contents
   '
   For Each mnu In mnuArray
      If mnu.Index > 0 Then
         Unload mnu
      End If
   Next mnu
   '
   ' Find out how many formats are present, and
   ' expand menu array to accomodate.
   '
   nCnt = CountClipboardFormats
   If nCnt = 0 Then
      mnuArray(0).Caption = "Empty"
      mnuArray(0).Enabled = False
   ElseIf nCnt >= 1 Then
      mnuArray(0).Enabled = True
      For i = 2 To nCnt
         Load mnuArray(i - 1)
         mnuArray(i - 1).Enabled = True
      Next i
   End If
   '
   ' Open and Enumerate clipboard formats.
   ' wFormat param should be 0 on first call to EnumClipboardFormats.
   '
   If OpenClipboard(0&) Then
      sFmt = Space(256)
      nCnt = 0
      Do
         nFmt = EnumClipboardFormats(nFmt)
         If nFmt Then
            Set mnu = mnuArray(nCnt)
            mnu.Caption = GetClipFormatName(nFmt)
            mnu.Tag = nFmt
            nCnt = nCnt + 1
         End If
      Loop While nFmt
      Call CloseClipboard
   End If
End Sub

Public Function GetClipFormatName(ByVal nFmt As Long)
   Dim nRet
   Dim sFmt As String
   Select Case nFmt
      Case CF_TEXT
         sFmt = "Text (TXT)"
      Case CF_BITMAP
         sFmt = "Bitmap (BMP)"
      Case CF_METAFILEPICT
         sFmt = "Metafile (WMF)"
      Case CF_SYLK
         sFmt = "Microsoft Symbolic Link (SYLK)"
      Case CF_DIF
         sFmt = "Data Interchange Format (DIF)"
      Case CF_TIFF
         sFmt = "Tagged Interchange File Format (TIF)"
      Case CF_OEMTEXT
         sFmt = "OEM Text"
      Case CF_DIB
         sFmt = "Device Independent Bitmap (DIB)"
      Case CF_PALETTE
         sFmt = "Palette"
      Case CF_PENDATA
         sFmt = "Pen Data"
      Case CF_RIFF
         sFmt = "RIFF"
      Case CF_WAVE
         sFmt = "Wave"
      Case CF_UNICODETEXT
         sFmt = "Unicode Text"
      Case CF_ENHMETAFILE
         sFmt = "Enhanced Metafile (EMF)"
      Case CF_HDROP
         sFmt = "Dropped Filelist"
      Case CF_LOCALE
         sFmt = "Locale Identifier"
      Case Else
         sFmt = Space(256)
         nRet = GetClipboardFormatName(nFmt, sFmt, Len(sFmt))
         If nRet = 0 Then
            sFmt = "Unknown: " & CStr(nFmt)
         Else
            sFmt = Left$(sFmt, nRet) & "  (" & CStr(nFmt) & ")"
         End If
   End Select
   GetClipFormatName = sFmt
End Function

' ****************************************************
'  Clipboard manipulation routines (private)
' ****************************************************
Private Function PaintPalette(pic As PictureBox) As Boolean
   Dim i As Long, j As Long
   
   With pic
      .Picture = Clipboard.GetData(vbCFPalette)
      .ScaleWidth = 16
      .ScaleHeight = 16
      For i = 0 To 15
         For j = 0 To 15
            pic.Line (j, i)-(j + 1, i + 1), &H1000000 + (j + i * 16), BF
         Next j
      Next i
   End With
   PaintPalette = True
End Function

Private Function PaintEnhMetafile(pic As PictureBox) As Boolean
   Dim hEmf As Long
   Dim emh As ENHMETAHEADER
   
   If OpenClipboard(0&) Then
      hEmf = GetClipboardData(CF_ENHMETAFILE)
      Call GetEnhMetaFileHeader(hEmf, Len(emh), emh)
      PaintEnhMetafile = PlayEnhMetaFile(pic.hdc, hEmf, emh.rclBounds)
      Call CloseClipboard
      pic.Refresh
   End If
End Function

Private Function PaintMetafile(pic As PictureBox) As Boolean
   pic.Picture = Clipboard.GetData(vbCFMetafile)
   PaintMetafile = True
   
'   Dim mfp As METAFILEPICT
'   Dim lpMfp As Long
'   Dim rClient As RECT
'
'   If OpenClipboard(0&) Then
'      lpMfp = GetClipboardData(CF_METAFILEPICT)
'      If lpMfp Then
'         CopyMem mfp, ByVal lpMfp, Len(mfp)
'         With pic
'            .ScaleMode = vbPixels
'            Call SetMapMode(.hDC, MM_ANISOTROPIC)
'            Call SetWindowExtEx(.hDC, .ScaleWidth, .ScaleHeight, ByVal 0&)     'arbitrary (?)
'            Call SetWindowOrgEx(.hDC, 0, 0, ByVal 0&)
'            Call GetClientRect(.hWnd, rClient)
'            Call SetViewportExtEx(.hDC, rClient.Right, rClient.Bottom, ByVal 0&)
'            If PlayMetaFile(.Image, mfp.hMF) Then
'               PaintMetafile = True
'            Else
'               PaintMetafile = False
'               Debug.Print "PlayMetafile Error: " & Err.LastDllError
'            End If
'            .Refresh
'         End With
'      Else
'         Debug.Print "GetClipboardData Error: " & Err.LastDllError
'      End If
'      Call CloseClipboard
'      pic.Refresh
'   End If
End Function

