Attribute VB_Name = "MLocaleInfo"
' *************************************************************
'  Copyright (C)1995-98, Karl E. Peterson
'  Author grants royalty-free rights to use this code within
'  compiled applications. Selling or otherwise distributing
'  this source code is not allowed without author's express
'  permission.
' *************************************************************
Option Explicit

' Locale Dependent APIs

Private Declare Function CompareString Lib "kernel32" Alias "CompareStringA" (ByVal Locale As Long, ByVal dwCmpFlags As Long, ByVal lpString1 As String, ByVal cchCount1 As Long, ByVal lpString2 As String, ByVal cchCount2 As Long) As Long
Private Declare Function LCMapString Lib "kernel32" Alias "LCMapStringA" (ByVal Locale As Long, ByVal dwMapFlags As Long, ByVal lpSrcStr As String, ByVal cchSrc As Long, ByVal lpDestStr As String, ByVal cchDest As Long) As Long
Private Declare Function GetLocaleInfo Lib "kernel32" Alias "GetLocaleInfoA" (ByVal Locale As Long, ByVal LCType As Long, ByVal lpLCData As String, ByVal cchData As Long) As Long
Private Declare Function GetTimeFormat Lib "kernel32" Alias "GetTimeFormatA" (ByVal Locale As Long, ByVal dwFlags As Long, lpTime As SYSTEMTIME, ByVal lpFormat As String, ByVal lpTimeStr As String, ByVal cchTime As Long) As Long
Private Declare Function GetDateFormat Lib "kernel32" Alias "GetDateFormatA" (ByVal Locale As Long, ByVal dwFlags As Long, lpDate As SYSTEMTIME, ByVal lpFormat As String, ByVal lpDateStr As String, ByVal cchDate As Long) As Long
Private Declare Function SetThreadLocale Lib "kernel32" (ByVal Locale As Long) As Long
Private Declare Function GetSystemDefaultLangID Lib "kernel32" () As Integer
Private Declare Function GetUserDefaultLangID Lib "kernel32" () As Integer
Private Declare Function GetSystemDefaultLCID Lib "kernel32" () As Long
Private Declare Function GetUserDefaultLCID Lib "kernel32" () As Long

' Locale Independent APIs

Private Declare Function GetStringTypeA Lib "kernel32" (ByVal LCID As Long, ByVal dwInfoType As Long, ByVal lpSrcStr As String, ByVal cchSrc As Long, lpCharType As Long) As Long
Private Declare Function FoldString Lib "kernel32" Alias "FoldStringA" (ByVal dwMapFlags As Long, ByVal lpSrcStr As String, ByVal cchSrc As Long, ByVal lpDestStr As String, ByVal cchDest As Long) As Long

' Locale Independent Mapping Flags.
Private Const MAP_FOLDCZONE = &H10        '  fold compatibility zone chars
Private Const MAP_PRECOMPOSED = &H20        '  convert to precomposed chars
Private Const MAP_COMPOSITE = &H40        '  convert to composite chars
Private Const MAP_FOLDDIGITS = &H80        '  all digits to ASCII 0-9

' Locale Dependent Mapping Flags.
Private Const LCMAP_LOWERCASE = &H100       '  lower case letters
Private Const LCMAP_UPPERCASE = &H200       '  upper case letters
Private Const LCMAP_SORTKEY = &H400       '  WC sort key (normalize)
Private Const LCMAP_BYTEREV = &H800       '  byte reversal

' Sorting Flags.
Private Const SORT_STRINGSORT = &H1000      '  use string sort method

' Code Page Default Values.
Private Const CP_ACP = 0  '  default to ANSI code page
Private Const CP_OEMCP = 1  '  default to OEM  code page

' Country Codes.

Private Const CTRY_DEFAULT = 0

Private Const CTRY_AUSTRALIA = 61  '  Australia
Private Const CTRY_AUSTRIA = 43  '  Austria
Private Const CTRY_BELGIUM = 32  '  Belgium
Private Const CTRY_BRAZIL = 55  '  Brazil
Private Const CTRY_CANADA = 2  '  Canada
Private Const CTRY_DENMARK = 45  '  Denmark
Private Const CTRY_FINLAND = 358  '  Finland
Private Const CTRY_FRANCE = 33  '  France
Private Const CTRY_GERMANY = 49  '  Germany
Private Const CTRY_ICELAND = 354  '  Iceland
Private Const CTRY_IRELAND = 353  '  Ireland
Private Const CTRY_ITALY = 39  '  Italy
Private Const CTRY_JAPAN = 81  '  Japan
Private Const CTRY_MEXICO = 52  '  Mexico
Private Const CTRY_NETHERLANDS = 31  '  Netherlands
Private Const CTRY_NEW_ZEALAND = 64  '  New Zealand
Private Const CTRY_NORWAY = 47  '  Norway
Private Const CTRY_PORTUGAL = 351  '  Portugal
Private Const CTRY_PRCHINA = 86  '  PR China
Private Const CTRY_SOUTH_KOREA = 82  '  South Korea
Private Const CTRY_SPAIN = 34  '  Spain
Private Const CTRY_SWEDEN = 46  '  Sweden
Private Const CTRY_SWITZERLAND = 41  '  Switzerland
Private Const CTRY_TAIWAN = 886  '  Taiwan
Private Const CTRY_UNITED_KINGDOM = 44  '  United Kingdom
Private Const CTRY_UNITED_STATES = 1  '  United States

' Locale Types.
' These types are used for the GetLocaleInfoW NLS API routine.

' LOCALE_NOUSEROVERRIDE is also used in GetTimeFormatW and GetDateFormatW.
Private Const LOCALE_NOUSEROVERRIDE = &H80000000  '  do not use user overrides

Private Const LOCALE_ILANGUAGE = &H1         '  language id
Private Const LOCALE_SLANGUAGE = &H2         '  localized name of language
Private Const LOCALE_SENGLANGUAGE = &H1001      '  English name of language
Private Const LOCALE_SABBREVLANGNAME = &H3         '  abbreviated language name
Private Const LOCALE_SNATIVELANGNAME = &H4         '  native name of language
Private Const LOCALE_ICOUNTRY = &H5         '  country code
Private Const LOCALE_SCOUNTRY = &H6         '  localized name of country
Private Const LOCALE_SENGCOUNTRY = &H1002      '  English name of country
Private Const LOCALE_SABBREVCTRYNAME = &H7         '  abbreviated country name
Private Const LOCALE_SNATIVECTRYNAME = &H8         '  native name of country
Private Const LOCALE_IDEFAULTLANGUAGE = &H9         '  default language id
Private Const LOCALE_IDEFAULTCOUNTRY = &HA         '  default country code
Private Const LOCALE_IDEFAULTCODEPAGE = &HB         '  default code page

Private Const LOCALE_SLIST = &HC         '  list item separator
Private Const LOCALE_IMEASURE = &HD         '  0 = metric, 1 = US

Private Const LOCALE_SDECIMAL = &HE         '  decimal separator
Private Const LOCALE_STHOUSAND = &HF         '  thousand separator
Private Const LOCALE_SGROUPING = &H10        '  digit grouping
Private Const LOCALE_IDIGITS = &H11        '  number of fractional digits
Private Const LOCALE_ILZERO = &H12        '  leading zeros for decimal
Private Const LOCALE_SNATIVEDIGITS = &H13        '  native ascii 0-9

Private Const LOCALE_SCURRENCY = &H14        '  local monetary symbol
Private Const LOCALE_SINTLSYMBOL = &H15        '  intl monetary symbol
Private Const LOCALE_SMONDECIMALSEP = &H16        '  monetary decimal separator
Private Const LOCALE_SMONTHOUSANDSEP = &H17        '  monetary thousand separator
Private Const LOCALE_SMONGROUPING = &H18        '  monetary grouping
Private Const LOCALE_ICURRDIGITS = &H19        '  # local monetary digits
Private Const LOCALE_IINTLCURRDIGITS = &H1A        '  # intl monetary digits
Private Const LOCALE_ICURRENCY = &H1B        '  positive currency mode
Private Const LOCALE_INEGCURR = &H1C        '  negative currency mode

Private Const LOCALE_SDATE = &H1D        '  date separator
Private Const LOCALE_STIME = &H1E        '  time separator
Private Const LOCALE_SSHORTDATE = &H1F        '  short date format string
Private Const LOCALE_SLONGDATE = &H20        '  long date format string
Private Const LOCALE_STIMEFORMAT = &H1003      '  time format string
Private Const LOCALE_IDATE = &H21        '  short date format ordering
Private Const LOCALE_ILDATE = &H22        '  long date format ordering
Private Const LOCALE_ITIME = &H23        '  time format specifier
Private Const LOCALE_ICENTURY = &H24        '  century format specifier
Private Const LOCALE_ITLZERO = &H25        '  leading zeros in time field
Private Const LOCALE_IDAYLZERO = &H26        '  leading zeros in day field
Private Const LOCALE_IMONLZERO = &H27        '  leading zeros in month field
Private Const LOCALE_S1159 = &H28        '  AM designator
Private Const LOCALE_S2359 = &H29        '  PM designator

Private Const LOCALE_SDAYNAME1 = &H2A        '  long name for Monday
Private Const LOCALE_SDAYNAME2 = &H2B        '  long name for Tuesday
Private Const LOCALE_SDAYNAME3 = &H2C        '  long name for Wednesday
Private Const LOCALE_SDAYNAME4 = &H2D        '  long name for Thursday
Private Const LOCALE_SDAYNAME5 = &H2E        '  long name for Friday
Private Const LOCALE_SDAYNAME6 = &H2F        '  long name for Saturday
Private Const LOCALE_SDAYNAME7 = &H30        '  long name for Sunday
Private Const LOCALE_SABBREVDAYNAME1 = &H31        '  abbreviated name for Monday
Private Const LOCALE_SABBREVDAYNAME2 = &H32        '  abbreviated name for Tuesday
Private Const LOCALE_SABBREVDAYNAME3 = &H33        '  abbreviated name for Wednesday
Private Const LOCALE_SABBREVDAYNAME4 = &H34        '  abbreviated name for Thursday
Private Const LOCALE_SABBREVDAYNAME5 = &H35        '  abbreviated name for Friday
Private Const LOCALE_SABBREVDAYNAME6 = &H36        '  abbreviated name for Saturday
Private Const LOCALE_SABBREVDAYNAME7 = &H37        '  abbreviated name for Sunday
Private Const LOCALE_SMONTHNAME1 = &H38        '  long name for January
Private Const LOCALE_SMONTHNAME2 = &H39        '  long name for February
Private Const LOCALE_SMONTHNAME3 = &H3A        '  long name for March
Private Const LOCALE_SMONTHNAME4 = &H3B        '  long name for April
Private Const LOCALE_SMONTHNAME5 = &H3C        '  long name for May
Private Const LOCALE_SMONTHNAME6 = &H3D        '  long name for June
Private Const LOCALE_SMONTHNAME7 = &H3E        '  long name for July
Private Const LOCALE_SMONTHNAME8 = &H3F        '  long name for August
Private Const LOCALE_SMONTHNAME9 = &H40        '  long name for September
Private Const LOCALE_SMONTHNAME10 = &H41        '  long name for October
Private Const LOCALE_SMONTHNAME11 = &H42        '  long name for November
Private Const LOCALE_SMONTHNAME12 = &H43        '  long name for December
Private Const LOCALE_SABBREVMONTHNAME1 = &H44        '  abbreviated name for January
Private Const LOCALE_SABBREVMONTHNAME2 = &H45        '  abbreviated name for February
Private Const LOCALE_SABBREVMONTHNAME3 = &H46        '  abbreviated name for March
Private Const LOCALE_SABBREVMONTHNAME4 = &H47        '  abbreviated name for April
Private Const LOCALE_SABBREVMONTHNAME5 = &H48        '  abbreviated name for May
Private Const LOCALE_SABBREVMONTHNAME6 = &H49        '  abbreviated name for June
Private Const LOCALE_SABBREVMONTHNAME7 = &H4A        '  abbreviated name for July
Private Const LOCALE_SABBREVMONTHNAME8 = &H4B        '  abbreviated name for August
Private Const LOCALE_SABBREVMONTHNAME9 = &H4C        '  abbreviated name for September
Private Const LOCALE_SABBREVMONTHNAME10 = &H4D        '  abbreviated name for October
Private Const LOCALE_SABBREVMONTHNAME11 = &H4E        '  abbreviated name for November
Private Const LOCALE_SABBREVMONTHNAME12 = &H4F        '  abbreviated name for December
Private Const LOCALE_SABBREVMONTHNAME13 = &H100F

Private Const LOCALE_SPOSITIVESIGN = &H50        '  positive sign
Private Const LOCALE_SNEGATIVESIGN = &H51        '  negative sign
Private Const LOCALE_IPOSSIGNPOSN = &H52        '  positive sign position
Private Const LOCALE_INEGSIGNPOSN = &H53        '  negative sign position
Private Const LOCALE_IPOSSYMPRECEDES = &H54        '  mon sym precedes pos amt
Private Const LOCALE_IPOSSEPBYSPACE = &H55        '  mon sym sep by space from pos amt
Private Const LOCALE_INEGSYMPRECEDES = &H56        '  mon sym precedes neg amt
Private Const LOCALE_INEGSEPBYSPACE = &H57        '  mon sym sep by space from neg amt

' Time Flags for GetTimeFormatW.
Private Const TIME_NOMINUTESORSECONDS = &H1         '  do not use minutes or seconds
Private Const TIME_NOSECONDS = &H2         '  do not use seconds
Private Const TIME_NOTIMEMARKER = &H4         '  do not use time marker
Private Const TIME_FORCE24HOURFORMAT = &H8         '  always use 24 hour format

' Date Flags for GetDateFormatW.
Private Const DATE_SHORTDATE = &H1         '  use short date picture
Private Const DATE_LONGDATE = &H2         '  use long date picture

Private Type SYSTEMTIME
        wYear As Integer
        wMonth As Integer
        wDayOfWeek As Integer
        wDay As Integer
        wHour As Integer
        wMinute As Integer
        wSecond As Integer
        wMilliseconds As Integer
End Type


Public Function LocaleDescLong(ByVal LCID As Long) As String
   Dim buf As String
   Dim desc As String
   Dim nRet As Long
   
   nRet = GetLocaleInfo(LCID, LOCALE_ILANGUAGE, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_ILANGUAGE, buf, Len(buf))
   desc = desc & "Locale ID: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SCOUNTRY, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SCOUNTRY, buf, Len(buf))
   desc = desc & "Country: " & TrimNull(buf) & "  "
   nRet = GetLocaleInfo(LCID, LOCALE_SABBREVCTRYNAME, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SABBREVCTRYNAME, buf, Len(buf))
   desc = desc & "(" & TrimNull(buf) & ")" & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SLANGUAGE, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SLANGUAGE, buf, Len(buf))
   desc = desc & "Language: " & TrimNull(buf) & vbCrLf & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SDECIMAL, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SDECIMAL, buf, Len(buf))
   desc = desc & "Decimal: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_STHOUSAND, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_STHOUSAND, buf, Len(buf))
   desc = desc & "Thousands: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SCURRENCY, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SCURRENCY, buf, Len(buf))
   desc = desc & "Currency: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_STIMEFORMAT, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_STIMEFORMAT, buf, Len(buf))
   desc = desc & "Time Format: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SSHORTDATE, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SSHORTDATE, buf, Len(buf))
   desc = desc & "Short Date: " & TrimNull(buf) & vbCrLf
   
   nRet = GetLocaleInfo(LCID, LOCALE_SLONGDATE, buf, 0)
   buf = Space$(nRet)
   Call GetLocaleInfo(LCID, LOCALE_SLONGDATE, buf, Len(buf))
   desc = desc & "Long Date: " & TrimNull(buf) & vbCrLf
   
   LocaleDescLong = desc
End Function



